@@ -1648,7 +1648,7 @@ float Falign_udpari_long( char  **seq1, char  **seq2,
 	int len1, len2;
 	int totallen;
 	float totalscore;
-	int nkouho;
+	int nkouho = 0;
 	int headgp, tailgp;
 //	float dumfl = 0.0;
 
