@@ -89,15 +89,17 @@ static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int
 #endif
 }
 
-static float Atracking( 
+static float Atracking( float *lasthorizontalw, float *lastverticalw,
 						char **seq1, char **seq2, 
                         char **mseq1, char **mseq2, 
                         int **ijp, int icyc, int jcyc,
-						int ist, int ien, int jst, int jen )
+						int ist, int ien, int jst, int jen, 
+						int fulllen1, int fulllen2, int tailgp )
 {
 	int i, j, l, iin, jin, ifi, jfi, lgth1, lgth2, k, klim;
 	char *gaptable1, *gt1bk;
 	char *gaptable2, *gt2bk;
+	float wm;
 	lgth1 = ien-ist+1;
 	lgth2 = jen-jst+1;
 
@@ -112,7 +114,63 @@ static float Atracking(
 #endif
 
 
-//	fprintf( stderr, "in Atracking, lgth1=%d, lgth2=%d\n", lgth1, lgth2 );
+//	fprintf( stderr, "in Atracking, tailgp=%d, ien=%d, fulllen1=%d, jen=%d, fulllen2=%d\n", tailgp, ien, fulllen1, jen, fulllen2 );
+
+	if( tailgp == 1 )
+		;
+	else if( ien == fulllen1-1 || jen == fulllen2-1 )
+	{
+//		fprintf( stderr, "searching lasthorizontalw and lasthorizontalw\n" );
+		wm = lastverticalw[0];
+		for( i=0; i<lgth1; i++ )
+		{
+			if( lastverticalw[i] >= wm )
+			{
+				wm = lastverticalw[i];
+				iin = i; jin = lgth2-1;
+				ijp[lgth1][lgth2] = +( lgth1 - i );
+			}
+		}
+		for( j=0; j<lgth2; j++ )
+		{
+			if( lasthorizontalw[j] >= wm )
+			{
+				wm = lasthorizontalw[j];
+				iin = lgth1-1; jin = j;
+				ijp[lgth1][lgth2] = -( lgth2 - j );
+			}
+		}
+	}
+#if 0
+	else if( jen == fulllen2-1 )
+	{
+		fprintf( stderr, "searching lastverticalw\n" );
+		wm = lastverticalw[0];
+		for( i=0; i<lgth1; i++ )
+		{
+			if( lastverticalw[i] >= wm )
+			{
+				wm = lastverticalw[i];
+				iin = i; jin = lgth2-1;
+				ijp[lgth1][lgth2] = +( lgth1 - i );
+			}
+		}
+	}
+	else if( ien == fulllen1-1 )
+	{
+		fprintf( stderr, "searching lasthorizontalw\n" );
+		wm = lasthorizontalw[0];
+		for( j=0; j<lgth2; j++ )
+		{
+			if( lasthorizontalw[j] >= wm )
+			{
+				wm = lasthorizontalw[j];
+				iin = lgth1-1; jin = j;
+				ijp[lgth1][lgth2] = -( lgth2 - j );
+			}
+		}
+	}
+#endif
  
     for( i=0; i<lgth1+1; i++ ) 
     {
@@ -180,7 +238,7 @@ static float Atracking(
 	return( 0.0 );
 }
 
-static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, char **seq1, char **seq2, float **cpmx1, float **cpmx2, int ist, int ien, int jst, int jen, int alloclen, char **mseq1, char **mseq2, float **gapinfo )
+static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, char **seq1, char **seq2, float **cpmx1, float **cpmx2, int ist, int ien, int jst, int jen, int alloclen, int fulllen1, int fulllen2, char **mseq1, char **mseq2, float **gapinfo, int headgp, int tailgp )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	int k;
@@ -217,6 +275,7 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 //	char **aseq2;
 //	char **aseq1bk, **aseq2bk;
 
+
 	ogcp1 = gapinfo[0] + ist;
 	fgcp1 = gapinfo[1] + ist;
 	ogcp2 = gapinfo[2] + jst;
@@ -240,6 +299,13 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 	fprintf( stderr, "ttt2 = %s\n", ttt2 );
 #endif
 
+#if 0
+	fprintf( stderr, "in _tanni ist,ien = %d,%d, fulllen1=%d\n", ist, ien, fulllen1 );
+	fprintf( stderr, "in _tanni jst,jen = %d,%d, fulllen2=%d\n", jst, jen, fulllen2 );
+	fprintf( stderr, "in _tanni seq1[0] = %-*.*s\n", ien-ist+1, ien-ist+1, seq1[0]+ist );
+	fprintf( stderr, "in _tanni seq2[0] = %-*.*s\n", jen-jst+1, jen-jst+1, seq2[0]+jst );
+#endif
+
 
 	ll1 = ( (int)(lgth1) ) + 100;
 	ll2 = ( (int)(lgth2) ) + 100;
@@ -275,13 +341,19 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 
 	match_calc( currentw, cpmx1+ist, cpmx2+jst, 0, lgth2, floatwork, intwork, 1 );
 
-	for( i=1; i<lgth1+1; i++ )
+	if( headgp || ist != 0 )
 	{
-		initverticalw[i] += ( ogcp1[0] + fgcp1[i-1] );
+		for( i=1; i<lgth1+1; i++ )
+		{
+			initverticalw[i] += ( ogcp1[0] + fgcp1[i-1] ) ;
+		}
 	}
-	for( j=1; j<lgth2+1; j++ )
+	if( headgp || jst != 0 )
 	{
-		currentw[j] += ( ogcp2[0] + fgcp2[j-1] );
+		for( j=1; j<lgth2+1; j++ )
+		{
+			currentw[j] += ( ogcp2[0] + fgcp2[j-1] ) ;
+		}
 	}
 
 	for( j=1; j<lgth2+1; ++j ) 
@@ -293,10 +365,10 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 
 
 
-	lasti = lgth1+1;
+	if( tailgp || jen != fulllen2-1 ) lasti = lgth1+1; else lasti = lgth1;
+//	if( 1 ) lasti = lgth1+1; else lasti = lgth1;
 	for( i=1; i<lasti; i++ )
 	{
-
 		wtmp = previousw; 
 		previousw = currentw;
 		currentw = wtmp;
@@ -314,7 +386,8 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 		prept = previousw;
 		curpt = currentw + 1;
 		mpjpt = mp + 1;
-		lastj = lgth2+1;
+//		if( tailgp && jen != fulllen2-1 ) lastj = lgth2+1; else lastj = lgth2;
+		lastj = lgth2+1; 
 		for( j=1; j<lastj; j++ )
 		{
 			wm = *prept;
@@ -378,7 +451,11 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 
 //	fprintf( stderr, "wm = %f\n", wm );
 
-	Atracking( seq1, seq2, mseq1, mseq2, ijp, icyc, jcyc, ist, ien, jst, jen );
+	Atracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, ijp, icyc, jcyc, ist, ien, jst, jen, fulllen1, fulllen2, tailgp );
+#if 0
+	fprintf( stderr, "res in _tanni mseq1[0] = %s\n", mseq1[0] );
+	fprintf( stderr, "res in _tanni mseq2[0] = %s\n", mseq2[0] );
+#endif
 
 //	for( i=0; i<icyc; i++ ) strcpy( mseq1[i], aseq1[i] );
 //	for( i=0; i<jcyc; i++ ) strcpy( mseq2[i], aseq2[i] );
@@ -400,6 +477,7 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 
 	FreeIntMtx( intmtx );
 
+
 //	FreeCharMtx( aseq1bk );
 //	FreeCharMtx( aseq2bk );
 
@@ -410,7 +488,58 @@ static float MSalignmm_tanni( int icyc, int jcyc, double *eff1, double *eff2, ch
 
 }
 
-static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char **seq1, char **seq2, float **cpmx1, float **cpmx2, int ist, int ien, int jst, int jen, int alloclen, char **mseq1, char **mseq2, int depth, float **gapinfo )
+static void freearrays_rec1(
+	float *w1, float *w2, float *initverticalw, float *lastverticalw,
+	float *midw, float *midm, float *midn,
+	int *jumpbacki, int *jumpbackj, int *jumpforwi, int *jumpforwj, int *jumpdummi, int *jumpdummj,
+	float *m, int *mp,
+	float **floatwork, int **intwork
+#if STOREWM
+	, float **WMMTX, float **WMMTX2
+#endif
+)
+{
+
+	FreeFloatVec( w1 );
+	FreeFloatVec( w2 );
+	FreeFloatVec( initverticalw );
+	FreeFloatVec( lastverticalw );
+	FreeFloatVec( midw );
+	FreeFloatVec( midm );
+	FreeFloatVec( midn );
+
+	FreeIntVec( jumpbacki );
+	FreeIntVec( jumpbackj );
+	FreeIntVec( jumpforwi );
+	FreeIntVec( jumpforwj );
+	FreeIntVec( jumpdummi );
+	FreeIntVec( jumpdummj );
+
+	FreeFloatVec( m );
+	FreeIntVec( mp );
+
+	FreeFloatMtx( floatwork );
+	FreeIntMtx( intwork );
+
+#if STOREWM
+	FreeFloatMtx( WMMTX );
+	FreeFloatMtx( WMMTX2 );
+#endif
+}
+
+static void freearrays_rec2( char *gaps, char **aseq1, char **aseq2 )
+{
+	free( gaps );
+#if MEMSAVE
+	free( aseq1 );
+	free( aseq2 );
+#else
+	FreeCharMtx( aseq1 );
+	FreeCharMtx( aseq2 );
+#endif
+}
+
+static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char **seq1, char **seq2, float **cpmx1, float **cpmx2, int ist, int ien, int jst, int jen, int alloclen, int fulllen1, int fulllen2, char **mseq1, char **mseq2, int depth, float **gapinfo, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	int k;
@@ -472,6 +601,14 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 	static TLS char ttt2[50000];
 #endif
 
+#if 0
+	int nglen1, nglen2;
+	nglen1 = seqlen( seq1[0] );
+	nglen2 = seqlen( seq2[0] );
+#endif
+
+//	fprintf( stderr, "fulllen1 = %d, fulllen2 = %d, headgp = %d, tailgp = %d\n", fulllen1, fulllen2, headgp, tailgp );
+
 	ogcp1 = gapinfo[0] + ist;
 	fgcp1 = gapinfo[1] + ist;
 	ogcp2 = gapinfo[2] + jst;
@@ -489,6 +626,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 //		fprintf( stderr, "\nWARNING: lgth2 = %d\n", lgth2 );
 //
 
+
 #if STOREWM
 	fprintf( stderr, "==== MSalign (depth=%d, reccycle=%d), ist=%d, ien=%d, jst=%d, jen=%d\n", depth, reccycle, ist, ien, jst, jen );
 	strncpy( ttt1, seq1[0]+ist, lgth1 );
@@ -530,13 +668,16 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 	aseq2 = AllocateCharMtx( jcyc, lgth1+lgth2+100 );
 #endif
 
+//	fprintf( stderr, "####(s) seq1[0] (%d) = \n%-*.*s\n (a%d-a%d)\n", depth, ien-ist+1, ien-ist+1, seq1[0]+ist, ist, ien );
+//	fprintf( stderr, "####(s) seq2[0] (%d) = \n%-*.*s\n (b%d-b%d)\n", depth, jen-jst+1, jen-jst+1, seq2[0]+jst, jst, jen );
+
 //  if( lgth1 < DPTANNI && lgth2 < DPTANNI ) // & dato lgth ==1 no kanousei ga arunode yokunai 
 //    if( lgth1 < DPTANNI ) // kore mo lgth2 ga mijikasugiru kanousei ari
     if( lgth1 < DPTANNI || lgth2 < DPTANNI ) // zettai ni anzen ka?
 	{
 //		fprintf( stderr, "==== Going to _tanni\n" );
 
-		value = MSalignmm_tanni( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, ist, ien, jst, jen, alloclen, aseq1, aseq2, gapinfo );	
+		value = MSalignmm_tanni( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, ist, ien, jst, jen, alloclen, fulllen1, fulllen2, aseq1, aseq2, gapinfo, headgp, tailgp );	
 
 
 #if MEMSAVE
@@ -611,11 +752,11 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 
 	for( i=1; i<lgth1+1; i++ )
 	{
-		initverticalw[i] += ( ogcp1[0] + fgcp1[i-1] );
+		initverticalw[i] += ( ogcp1[0] + fgcp1[i-1] ) ;
 	}
 	for( j=1; j<lgth2+1; j++ )
 	{
-		currentw[j] += ( ogcp2[0] + fgcp2[j-1] );
+		currentw[j] += ( ogcp2[0] + fgcp2[j-1] ) ;
 	}
 
 #if STOREWM
@@ -650,6 +791,26 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 	for( i=1; i<=imid; i++ )
 #endif
 	{
+#ifdef enablemultithread
+//		fprintf( stderr, "chudan = %d, %d\n", *chudanpt, chudanref );
+		if( chudanpt && *chudanpt != chudanref ) 
+		{
+//			fprintf( stderr, "\n\n## CHUUDAN!!! zenhan\n" );
+			*chudanres = 1;
+			freearrays_rec1
+			(
+				w1, w2, initverticalw, lastverticalw, midw, midm, midn,
+				jumpbacki, jumpbackj, jumpforwi, jumpforwj, jumpdummi, jumpdummj,
+				m, mp,
+				floatwork, intwork
+#if STOREWM
+				, WMMTX, WMMTX2
+#endif
+			);
+			freearrays_rec2( gaps, aseq1, aseq2 );
+			return( -1.0 );
+		}
+#endif
 		wtmp = previousw; 
 		previousw = currentw;
 		currentw = wtmp;
@@ -844,9 +1005,30 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 
 //	for( i=lgth1-2; i>=imid; i-- )
 	firstm = -9999999.9;
-	firstmp = lgth1-1;
+//	firstmp = lgth1-1;
+	firstmp = lgth1;
 	for( i=lgth1-2; i>-1; i-- )
 	{
+#ifdef enablemultithread
+//		fprintf( stderr, "chudan = %d, %d\n", *chudanpt, chudanref );
+		if( chudanpt && *chudanpt != chudanref ) 
+		{
+//			fprintf( stderr, "\n\n## CHUUDAN!!! kouhan\n" );
+			*chudanres = 1;
+			freearrays_rec1
+			(
+				w1, w2, initverticalw, lastverticalw, midw, midm, midn,
+				jumpbacki, jumpbackj, jumpforwi, jumpforwj, jumpdummi, jumpdummj,
+				m, mp,
+				floatwork, intwork
+#if STOREWM
+				, WMMTX, WMMTX2
+#endif
+			);
+			freearrays_rec2( gaps, aseq1, aseq2 );
+			return( -1.0 );
+		}
+#endif
 		wtmp = previousw;
 		previousw = currentw;
 		currentw = wtmp;
@@ -963,6 +1145,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 #endif
 		if( i == imid ) midm[j+1] += firstm;
 
+
 		if( i == imid - 1 )	
 		{
 			maxwm = midw[1];
@@ -1039,14 +1222,37 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 		}
 		if( i == jumpi ) //saki?
 		{
+//			fprintf( stderr, "#### FIRST i=%d, jumpi<imid=%d<%d, ist=%d, ien=%d, firstmp=%d, lgth1=%d\n", i, jumpi, imid, ist, ien, firstmp, lgth1 );
+//			fprintf( stderr, "#### mark 1\n" );
 //			fprintf( stderr, "imid, jumpi = %d,%d\n", imid, jumpi );
 //			fprintf( stderr, "jmid, jumpj = %d,%d\n", jmid, jumpj );
+
 			if( jmid == 0 )
 			{
-//				fprintf( stderr, "CHUI2!\n" );
+//				fprintf( stderr, "#### CHUI2!\n" );
 				jumpj = 0; jmid = 1;
-				jumpi = firstmp - 1;
+#if 0 // v6.823 made
+				jumpi = firstmp-1;
 				imid = firstmp;
+#endif
+#if 0
+				jumpi = 0;
+				imid = 1;
+#else
+//				if( 1 || firstmp > 100 ) // naze 100
+				if( imid < firstmp-1 ) // naze 100
+				{
+					jumpi = firstmp;
+					imid = firstmp+1;
+				}
+#if 0
+				else
+				{
+					jumpi = 0;
+					imid = 1;
+				}
+#endif
+#endif
 			}
 
 #if 0
@@ -1066,8 +1272,10 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 #endif
 			else
 			{
+//				fprintf( stderr, "#### CHUI3!\n" );
 				imid = jumpforwi[jumpj];
 				jmid = jumpforwj[jumpj];
+				if( imid == jumpi ) jumpi = imid-1;
 			}
 #if 0
 			fprintf( stderr, "jumpi -> %d\n", jumpi );
@@ -1075,6 +1283,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 			fprintf( stderr, "imid -> %d\n", imid );
 			fprintf( stderr, "jmid -> %d\n", jmid );
 #endif
+//			fprintf( stderr, "#### FINAL i=%d, jumpi<imid=%d<%d, ist=%d, ien=%d, firstmp=%d\n", i, jumpi, imid, ist, ien, firstmp );
 
 #if STOREWM
 			break;
@@ -1083,11 +1292,6 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 #endif
 		}
 	}
-#if 0
-		jumpi=0; jumpj=0;
-		imid=lgth1-1; jmid=lgth2-1;
-	}
-#endif
 
 //	fprintf( stderr, "imid = %d, but jumpi = %d\n", imid, jumpi );
 //	fprintf( stderr, "jmid = %d, but jumpj = %d\n", jmid, jumpj );
@@ -1170,40 +1374,21 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 	fprintf( stderr, "jmid = %d\n", jmid );
 #endif
 
-
-	FreeFloatVec( w1 );
-	FreeFloatVec( w2 );
-	FreeFloatVec( initverticalw );
-	FreeFloatVec( lastverticalw );
-	FreeFloatVec( midw );
-	FreeFloatVec( midm );
-	FreeFloatVec( midn );
-
-	FreeIntVec( jumpbacki );
-	FreeIntVec( jumpbackj );
-	FreeIntVec( jumpforwi );
-	FreeIntVec( jumpforwj );
-	FreeIntVec( jumpdummi );
-	FreeIntVec( jumpdummj );
-
-	FreeFloatVec( m );
-	FreeIntVec( mp );
-
-	FreeFloatMtx( floatwork );
-	FreeIntMtx( intwork );
-
+	freearrays_rec1
+	(
+		w1, w2, initverticalw, lastverticalw, midw, midm, midn,
+		jumpbacki, jumpbackj, jumpforwi, jumpforwj, jumpdummi, jumpdummj,
+		m, mp,
+		floatwork, intwork
 #if STOREWM
-	FreeFloatMtx( WMMTX );
-	FreeFloatMtx( WMMTX2 );
+		, WMMTX, WMMTX2
 #endif
+	);
+
 
 //	fprintf( stderr, "==== calling myself (first)\n" );
 
-#if 0
-		fprintf( stderr, "seq1[0] = %s\n", seq1[0] );
-		fprintf( stderr, "seq2[0] = %s\n", seq2[0] );
-#endif
-	value = MSalignmm_rec( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, ist, ist+jumpi, jst, jst+jumpj, alloclen, aseq1, aseq2, depth, gapinfo );	
+	value = MSalignmm_rec( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, ist, ist+jumpi, jst, jst+jumpj, alloclen, fulllen1, fulllen2, aseq1, aseq2, depth, gapinfo, NULL, 0, NULL, headgp, tailgp ); // chudan mada	
 #if 0
 		fprintf( stderr, "aseq1[0] = %s\n", aseq1[0] );
 		fprintf( stderr, "aseq2[0] = %s\n", aseq2[0] );
@@ -1276,11 +1461,7 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 	for( i=0; i<jcyc; i++ ) aseq2[i] += alnlen;
 #endif
 
-#if 0
-		fprintf( stderr, "seq1[0] = %s\n", seq1[0] );
-		fprintf( stderr, "seq2[0] = %s\n", seq2[0] );
-#endif
-	value += MSalignmm_rec( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, ist+imid, ien, jst+jmid, jen, alloclen, aseq1, aseq2, depth, gapinfo );	
+	value += MSalignmm_rec( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, ist+imid, ien, jst+jmid, jen, alloclen, fulllen1, fulllen2, aseq1, aseq2, depth, gapinfo, NULL, 0, NULL, headgp, tailgp ); // chudan mada
 #if 0
 		fprintf( stderr, "aseq1[0] = %s\n", aseq1[0] );
 		fprintf( stderr, "aseq2[0] = %s\n", aseq2[0] );
@@ -1317,24 +1498,57 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 	for( i=0; i<jcyc; i++ ) strcat( mseq2[i], aseq2[i] );
 #endif
 
-//	fprintf( stderr, "====(s) aseq1[0] (%d) = %s (%d-%d)\n", depth, aseq1[0], ist, ien );
-//	fprintf( stderr, "====(s) aseq2[0] (%d) = %s (%d-%d)\n", depth, aseq2[0], jst, jen );
+//	fprintf( stderr, "====(s) aseq1[0] (%d) = \n%s\n (a%d-a%d)\n", depth, mseq1[0], ist, ien );
+//	fprintf( stderr, "====(s) aseq2[0] (%d) = \n%s\n (b%d-b%d)\n", depth, mseq2[0], jst, jen );
 
-	free( gaps );
-#if MEMSAVE
-	free( aseq1 );
-	free( aseq2 );
-#else
-	FreeCharMtx( aseq1 );
-	FreeCharMtx( aseq2 );
+	freearrays_rec2( gaps, aseq1, aseq2 );
+
+#if 0
+	if( seqlen( seq1[0] ) != nglen1 )
+	{
+		fprintf( stderr, "bug! hairetsu ga kowareta! (nglen1) seqlen(seq1[0])=%d but nglen1=%d\n", seqlen( seq1[0] ), nglen1 );
+		fprintf( stderr, "seq1[0] = %s\n", seq1[0] );
+		exit( 1 );
+	}
+	else
+		fprintf( stderr, "nglen1 is ok in _rec\n" );
+	if( seqlen( seq2[0] ) != nglen2 )
+	{
+		fprintf( stderr, "bug! hairetsu ga kowareta! (nglen2) seqlen(seq2[0])=%d but nglen2=%d\n", seqlen( seq2[0] ), nglen2 );
+		exit( 1 );
+	}
+	else
+		fprintf( stderr, "nglen2 is ok in _rec\n" );
 #endif
-	
+
 	return( value );
 }
 
+static void freearrays( 
+	float *ogcp1, 
+	float *ogcp2, 
+	float *fgcp1, 
+	float *fgcp2, 
+	float **cpmx1,
+	float **cpmx2, 
+	float **gapinfo, 
+	char **mseq1, 
+	char **mseq2 
+)
+{
+	FreeFloatVec( ogcp1 );
+	FreeFloatVec( ogcp2 );
+	FreeFloatVec( fgcp1 );
+	FreeFloatVec( fgcp2 );
+	FreeFloatMtx( cpmx1 );
+	FreeFloatMtx( cpmx2 );
+	free( (void *)gapinfo );
 
+	FreeCharMtx( mseq1 );
+	FreeCharMtx( mseq2 );
+}
 
-float MSalignmm( char **seq1, char **seq2, double *eff1, double *eff2, int icyc, int jcyc, int alloclen, char *sgap1, char *sgap2, char *egap1, char *egap2 )
+float MSalignmm( char **seq1, char **seq2, double *eff1, double *eff2, int icyc, int jcyc, int alloclen, char *sgap1, char *sgap2, char *egap1, char *egap2, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	int k;
@@ -1362,6 +1576,13 @@ float MSalignmm( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	nglen1 = seqlen( seq1[0] );
 	nglen2 = seqlen( seq2[0] );
 
+#if 0
+	fprintf( stderr, "\n" );
+	for( i=0; i<icyc; i++ ) fprintf( stderr, "seq1[%d] at root = %s\n", i, seq1[i] );
+	for( j=0; j<jcyc; j++ ) fprintf( stderr, "seq2[%d] at root = %s\n", j, seq2[j] );
+	fprintf( stderr, "\n" );
+#endif
+
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
 
@@ -1468,7 +1689,17 @@ float MSalignmm( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	fflush( stdout );
 #endif
 
-	wm = MSalignmm_rec( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, 0, lgth1-1, 0, lgth2-1, alloclen, mseq1, mseq2, 0, gapinfo );
+	wm = MSalignmm_rec( icyc, jcyc, eff1, eff2, seq1, seq2, cpmx1, cpmx2, 0, lgth1-1, 0, lgth2-1, alloclen, lgth1, lgth2, mseq1, mseq2, 0, gapinfo, chudanpt, chudanref, chudanres, headgp, tailgp );
+#ifdef enablemultithread
+	if( chudanres && *chudanres ) 
+	{
+//		fprintf( stderr, "\n\n## CHUUDAN!!! relay\n" );
+		*chudanres = 1;
+		freearrays( ogcp1, ogcp2, fgcp1, fgcp2, cpmx1, cpmx2, gapinfo, mseq1, mseq2 );
+		return( -1.0 );
+	}
+#endif
+
 #if DEBUG
 		fprintf( stderr, " seq1[0] = %s\n", seq1[0] );
 		fprintf( stderr, " seq2[0] = %s\n", seq2[0] );
@@ -1494,16 +1725,8 @@ float MSalignmm( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 		exit( 1 );
 	}
 
-	FreeFloatVec( ogcp1 );
-	FreeFloatVec( ogcp2 );
-	FreeFloatVec( fgcp1 );
-	FreeFloatVec( fgcp2 );
-	FreeFloatMtx( cpmx1 );
-	FreeFloatMtx( cpmx2 );
-	free( (void *)gapinfo );
 
-	FreeCharMtx( mseq1 );
-	FreeCharMtx( mseq2 );
+	freearrays( ogcp1, ogcp2, fgcp1, fgcp2, cpmx1, cpmx2, gapinfo, mseq1, mseq2 );
 
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
