@@ -10,8 +10,8 @@
 
 
 
-static float **impmtx = NULL;
-static int impalloclen = 0;
+static TLS float **impmtx = NULL;
+static TLS int impalloclen = 0;
 #if 1 // tditeration to naiveQscore_imp de tsukawareru.
 float imp_match_out_scQ( int i1, int j1 )
 {
@@ -83,8 +83,8 @@ void imp_match_init_strictQ( float *imp, int clus1, int clus2, int lgth1, int lg
 	float effij;
 	double effijx;
 	char *pt, *pt1, *pt2;
-	static char *nocount1 = NULL;
-	static char *nocount2 = NULL;
+	static TLS char *nocount1 = NULL;
+	static TLS char *nocount2 = NULL;
 	LocalHom *tmpptr;
 
 	if( impalloclen < lgth1 + 2 || impalloclen < lgth2 + 2 )
@@ -391,8 +391,10 @@ static void Atracking_localhom_gapmap( float *impwmpt, float *lasthorizontalw, f
 						int *gapmap1, int *gapmap2 )
 {
 	int i, j, l, iin, jin, ifi, jfi, lgth1, lgth2, k;
-	char gap[] = "-";
+//	char gap[] = "-";
+	char *gap;
 	float wm;
+	gap = newgapstr;
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
 
@@ -859,43 +861,43 @@ float Q__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	float *mjpt, *prept, *curpt;
 	int *mpjpt;
 #endif
-	static float mi, *m;
-	static int **ijp;
-	static int mpi, *mp;
-	static float *w1, *w2;
-	static float *match;
-	static float *initverticalw;    /* kufuu sureba iranai */
-	static float *lastverticalw;    /* kufuu sureba iranai */
-	static char **mseq1;
-	static char **mseq2;
-	static char **mseq;
-	static float *digf1;
-	static float *digf2;
-	static float *diaf1;
-	static float *diaf2;
-	static float *gapz1;
-	static float *gapz2;
-	static float *gapf1;
-	static float *gapf2;
-	static float *ogcp1g;
-	static float *ogcp2g;
-	static float *fgcp1g;
-	static float *fgcp2g;
-	static float *og_h_dg_n1_p;
-	static float *og_h_dg_n2_p;
-	static float *fg_h_dg_n1_p;
-	static float *fg_h_dg_n2_p;
-	static float *og_t_fg_h_dg_n1_p;
-	static float *og_t_fg_h_dg_n2_p;
-	static float *fg_t_og_h_dg_n1_p;
-	static float *fg_t_og_h_dg_n2_p;
-	static float *gapz_n1;
-	static float *gapz_n2;
-	static float **cpmx1;
-	static float **cpmx2;
-	static int **intwork;
-	static float **floatwork;
-	static int orlgth1 = 0, orlgth2 = 0;
+	static TLS float mi, *m;
+	static TLS int **ijp;
+	static TLS int mpi, *mp;
+	static TLS float *w1, *w2;
+	static TLS float *match;
+	static TLS float *initverticalw;    /* kufuu sureba iranai */
+	static TLS float *lastverticalw;    /* kufuu sureba iranai */
+	static TLS char **mseq1;
+	static TLS char **mseq2;
+	static TLS char **mseq;
+	static TLS float *digf1;
+	static TLS float *digf2;
+	static TLS float *diaf1;
+	static TLS float *diaf2;
+	static TLS float *gapz1;
+	static TLS float *gapz2;
+	static TLS float *gapf1;
+	static TLS float *gapf2;
+	static TLS float *ogcp1g;
+	static TLS float *ogcp2g;
+	static TLS float *fgcp1g;
+	static TLS float *fgcp2g;
+	static TLS float *og_h_dg_n1_p;
+	static TLS float *og_h_dg_n2_p;
+	static TLS float *fg_h_dg_n1_p;
+	static TLS float *fg_h_dg_n2_p;
+	static TLS float *og_t_fg_h_dg_n1_p;
+	static TLS float *og_t_fg_h_dg_n2_p;
+	static TLS float *fg_t_og_h_dg_n1_p;
+	static TLS float *fg_t_og_h_dg_n2_p;
+	static TLS float *gapz_n1;
+	static TLS float *gapz_n2;
+	static TLS float **cpmx1;
+	static TLS float **cpmx2;
+	static TLS int **intwork;
+	static TLS float **floatwork;
+	static TLS int orlgth1 = 0, orlgth2 = 0;
 	float tmppenal;
 	float *fg_t_og_h_dg_n2_p_pt;
 	float *og_t_fg_h_dg_n2_p_pt;
@@ -1532,43 +1534,43 @@ float Q__align_gapmap( char **seq1, char **seq2, double *eff1, double *eff2, int
 	float *mjpt, *prept, *curpt;
 	int *mpjpt;
 #endif
-	static float mi, *m;
-	static int **ijp;
-	static int mpi, *mp;
-	static float *w1, *w2;
-	static float *match;
-	static float *initverticalw;    /* kufuu sureba iranai */
-	static float *lastverticalw;    /* kufuu sureba iranai */
-	static char **mseq1;
-	static char **mseq2;
-	static char **mseq;
-	static float *digf1;
-	static float *digf2;
-	static float *diaf1;
-	static float *diaf2;
-	static float *gapz1;
-	static float *gapz2;
-	static float *gapf1;
-	static float *gapf2;
-	static float *ogcp1g;
-	static float *ogcp2g;
-	static float *fgcp1g;
-	static float *fgcp2g;
-	static float *og_h_dg_n1_p;
-	static float *og_h_dg_n2_p;
-	static float *fg_h_dg_n1_p;
-	static float *fg_h_dg_n2_p;
-	static float *og_t_fg_h_dg_n1_p;
-	static float *og_t_fg_h_dg_n2_p;
-	static float *fg_t_og_h_dg_n1_p;
-	static float *fg_t_og_h_dg_n2_p;
-	static float *gapz_n1;
-	static float *gapz_n2;
-	static float **cpmx1;
-	static float **cpmx2;
-	static int **intwork;
-	static float **floatwork;
-	static int orlgth1 = 0, orlgth2 = 0;
+	static TLS float mi, *m;
+	static TLS int **ijp;
+	static TLS int mpi, *mp;
+	static TLS float *w1, *w2;
+	static TLS float *match;
+	static TLS float *initverticalw;    /* kufuu sureba iranai */
+	static TLS float *lastverticalw;    /* kufuu sureba iranai */
+	static TLS char **mseq1;
+	static TLS char **mseq2;
+	static TLS char **mseq;
+	static TLS float *digf1;
+	static TLS float *digf2;
+	static TLS float *diaf1;
+	static TLS float *diaf2;
+	static TLS float *gapz1;
+	static TLS float *gapz2;
+	static TLS float *gapf1;
+	static TLS float *gapf2;
+	static TLS float *ogcp1g;
+	static TLS float *ogcp2g;
+	static TLS float *fgcp1g;
+	static TLS float *fgcp2g;
+	static TLS float *og_h_dg_n1_p;
+	static TLS float *og_h_dg_n2_p;
+	static TLS float *fg_h_dg_n1_p;
+	static TLS float *fg_h_dg_n2_p;
+	static TLS float *og_t_fg_h_dg_n1_p;
+	static TLS float *og_t_fg_h_dg_n2_p;
+	static TLS float *fg_t_og_h_dg_n1_p;
+	static TLS float *fg_t_og_h_dg_n2_p;
+	static TLS float *gapz_n1;
+	static TLS float *gapz_n2;
+	static TLS float **cpmx1;
+	static TLS float **cpmx2;
+	static TLS int **intwork;
+	static TLS float **floatwork;
+	static TLS int orlgth1 = 0, orlgth2 = 0;
 	float tmppenal;
 	float *fg_t_og_h_dg_n2_p_pt;
 	float *og_t_fg_h_dg_n2_p_pt;
