@@ -967,6 +967,40 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 		return( 0.0 );
 	}
 
+	lgth1 = strlen( seq1[0] );
+	lgth2 = strlen( seq2[0] );
+#if 1
+	if( lgth1 == 0 || lgth2 == 0 )
+	{
+		fprintf( stderr, "WARNING (Aalignmm): lgth1=%d, lgth2=%d\n", lgth1, lgth2 );
+	}
+	if( lgth1 == 0 && lgth2 == 0 )
+		return( 0.0 );
+
+	if( lgth1 == 0 )
+	{
+		for( i=0; i<icyc; i++ )
+		{
+			j = lgth2;
+			seq1[i][j] = 0;
+			while( j ) seq1[i][--j] = '-';
+//			fprintf( stderr, "seq1[i] = %s\n", seq1[i] );
+		}
+		return( 0.0 );
+	}
+
+	if( lgth2 == 0 )
+	{
+		for( i=0; i<jcyc; i++ )
+		{
+			j = lgth1;
+			seq2[i][j] = 0;
+			while( j ) seq2[i][--j] = '-';
+//			fprintf( stderr, "seq2[i] = %s\n", seq2[i] );
+		}
+		return( 0.0 );
+	}
+#endif
 
 
 #if 0
@@ -985,14 +1019,6 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	}
 
 
-	lgth1 = strlen( seq1[0] );
-	lgth2 = strlen( seq2[0] );
-#if 0
-	if( lgth1 == 0 || lgth2 == 0 )
-	{
-		fprintf( stderr, "WARNING (Aalignmm): lgth1=%d, lgth2=%d\n", lgth1, lgth2 );
-	}
-#endif
 
 	if( lgth1 > orlgth1 || lgth2 > orlgth2 )
 	{
