@@ -64,11 +64,12 @@ void findnewgaps( int n, char **seq, int *gaplen )
 
 void findcommongaps( int n, char **seq, int *gaplen )
 {
-	int i, j, pos, len;
+	int i, j, pos, len, len1;
 	len = strlen( seq[0] );	
+	len1 = len+1;
 
 //	fprintf( stderr, "seq[0] = %s\n", seq[0] );
-	for( i=0; i<len; i++ ) gaplen[i] = 0;
+	for( i=0; i<len1; i++ ) gaplen[i] = 0;
 	
 	pos = 0;
 	for( i=0; i<len; i++ )
@@ -280,6 +281,9 @@ void restorecommongaps( int njob, char **seq, int *ex1, int *ex2, int *gaplen, i
 	mar = calloc( njob, sizeof( int ) );
 	tmpseq = calloc( alloclen, sizeof( char ) );
 	tmpgaplen = calloc( alloclen, sizeof( int ) );
+//	tmpseq = calloc( alloclen+2, sizeof( char ) );
+//	tmpgaplen = calloc( alloclen+2, sizeof( int ) );
+
 
 	for( i=0; i<njob; i++ ) mar[i] = 0;
 	for( i=0; (k=ex1[i])>-1; i++ ) 
@@ -308,6 +312,7 @@ void restorecommongaps( int njob, char **seq, int *ex1, int *ex2, int *gaplen, i
 	len = strlen( seq[rep] );
 	len1 = len+1;
 
+
 	for( i=0; i<njob; i++ )
 	{
 		if( mar[i] == 0 ) continue;
@@ -320,7 +325,6 @@ void restorecommongaps( int njob, char **seq, int *ex1, int *ex2, int *gaplen, i
 		}
 		*cptr = 0;
 		strcpy( seq[i], tmpseq );
-//		fprintf( stderr, "seq[%d] = %s\n", i, seq[i] );
 	}
 
 	iptr = tmpgaplen;
