@@ -351,7 +351,7 @@ void seq_grp_nuc( int *grp, char *seq )
 	*grp = END_OF_VEC;
 	if( grp - grpbk < 6 )
 	{
-		fprintf( stderr, "\n\nWARNING: Too short.\nPlease also consider use mafft-ginsi, mafft-linsi or mafft-ginsi.\n\n\n" );
+//		fprintf( stderr, "\n\nWARNING: Too short.\nPlease also consider use mafft-ginsi, mafft-linsi or mafft-ginsi.\n\n\n" );
 //		exit( 1 );
 		*grpbk = -1;
 	}
@@ -372,7 +372,7 @@ void seq_grp( int *grp, char *seq )
 	*grp = END_OF_VEC;
 	if( grp - grpbk < 6 )
 	{
-		fprintf( stderr, "\n\nWARNING: Too short.\nPlease also consider use mafft-ginsi, mafft-linsi or mafft-ginsi.\n\n\n" );
+//		fprintf( stderr, "\n\nWARNING: Too short.\nPlease also consider use mafft-ginsi, mafft-linsi or mafft-ginsi.\n\n\n" );
 //		exit( 1 );
 		*grpbk = -1;
 	}
@@ -627,7 +627,7 @@ static void *treebasethread( void *arg )
 
 		len1 = strlen( aseq[m1] );
 		len2 = strlen( aseq[m2] );
-		if( *alloclen < len1 + len2 )
+		if( *alloclen <= len1 + len2 )
 		{
 			fprintf( stderr, "\nReallocating.." );
 			*alloclen = ( len1 + len2 ) + 1000;
@@ -1269,9 +1269,9 @@ int main( int argc, char *argv[] )
 	getnumlen( infp );
 	rewind( infp );
 
-	if( njob > 20000 )
+	if( njob > 100000 )
 	{
-		fprintf( stderr, "The number of sequences must be < %d\n", 20000 );
+		fprintf( stderr, "The number of sequences must be < %d\n", 100000 );
 		fprintf( stderr, "Please try the --parttree option for such large data.\n" );
 		exit( 1 );
 	}
@@ -1577,7 +1577,7 @@ int main( int argc, char *argv[] )
 	FreeFloatMtx( len );
 
 	bseq = AllocateCharMtx( njob, nlenmax*2+1 );
-	alloclen = nlenmax*2;
+	alloclen = nlenmax*2+1;
 
 
 	if( nadd )
