@@ -142,7 +142,8 @@ static void *athread( void *arg )
 		if( system( com ) )
 		{
 			fprintf( stderr, "retrying to rmdir\n" );
-			usleep( 100000 );
+//			nanosleep( 100000 );
+			sleep( 1 );
 			system( com );
 		}
 	}
