@@ -647,7 +647,7 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 #if 1
 			if( use_fft )
 			{
-				pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum );
+				pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum, NULL, 0, NULL );
 				off1 = off2 = 0;
 			}
 			else
@@ -660,7 +660,7 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 						off1 = off2 = 0;
 						break;
 					case( 'A' ):
-						pscore = G__align11( mseq1, mseq2, alloclen );
+						pscore = G__align11( mseq1, mseq2, alloclen, NULL, 0, NULL );
 						off1 = off2 = 0;
 						break;
 #if 0
