@@ -830,6 +830,39 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 		return( 0.0 );
 	}
 
+	lgth1 = strlen( seq1[0] );
+	lgth2 = strlen( seq2[0] );
+#if 1
+	if( lgth1 == 0 ) fprintf( stderr, "WARNING: lgth1=0 in partA__align\n" );
+	if( lgth2 == 0 ) fprintf( stderr, "WARNING: lgth2=0 in partA__align\n" );
+
+	if( lgth1 == 0 && lgth2 == 0 )
+		return( 0.0 );
+
+	if( lgth1 == 0 )
+	{
+		for( i=0; i<icyc; i++ )
+		{
+			j = lgth2;
+			seq1[i][j] = 0;
+			while( j ) seq1[i][--j] = '-';
+//			fprintf( stderr, "seq1[i] = %s\n", seq1[i] );
+		}
+		return( 0.0 );
+	}
+
+	if( lgth2 == 0 )
+	{
+		for( i=0; i<jcyc; i++ )
+		{
+			j = lgth1;
+			seq2[i][j] = 0;
+			while( j ) seq2[i][--j] = '-';
+//			fprintf( stderr, "seq2[i] = %s\n", seq2[i] );
+		}
+		return( 0.0 );
+	}
+#endif
 
 #if 0
 	fprintf( stderr, "eff in SA+++align\n" );
@@ -842,8 +875,7 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 	}
 
 
-	lgth1 = strlen( seq1[0] );
-	lgth2 = strlen( seq2[0] );
+
 
 	if( lgth1 > orlgth1 || lgth2 > orlgth2 )
 	{
