@@ -2443,7 +2443,7 @@ exit( 1 );
 	{
 		int *mem1p, *mem2p;
 		int mem1size, mem2size;
-		int v1, v2, v3;
+		int v1 = 0, v2 = 0, v3 = 0;
 		int nlim;
 		int l;
 		static int *mem1 = NULL;
