@@ -583,7 +583,7 @@ static void *treebasethread( void *arg )
 
         len1 = strlen( aseq[m1] );
         len2 = strlen( aseq[m2] );
-        if( *alloclen < len1 + len2 )
+        if( *alloclen <= len1 + len2 )
         {
 			fprintf( stderr, "\nReallocating (by thread %d) ..", thread_no );
 			*alloclen = ( len1 + len2 ) + 1000;
@@ -1609,8 +1609,8 @@ int main( int argc, char *argv[] )
 	FreeFloatHalfMtx( iscore, njob );
 	FreeFloatMtx( len );
 
-	alloclen = nlenmax*2; //chuui!
-	bseq = AllocateCharMtx( njob, alloclen+1 );
+	alloclen = nlenmax*2+1; //chuui!
+	bseq = AllocateCharMtx( njob, alloclen );
 
 	if( nadd )
 	{
